use serde::Deserialize;

/// Holds the individual config segments.
#[derive(Debug, Clone, Deserialize, Default)]
pub struct AppConfig {
    pub server: ServerConfig,
    pub database: DatabaseConfig,
    pub logging: LoggingConfig,
    pub meshconfig: MeshConfig,
}

#[derive(Debug, Clone, Deserialize)]
pub struct ServerConfig {
    pub host: String,
    pub port: u16,
}

/// Holds the config for the database
#[derive(Debug, Clone, Deserialize)]
pub struct DatabaseConfig {
    pub db_name: String,
}

/// Holds the config for logging
#[derive(Debug, Clone, Deserialize)]
pub struct LoggingConfig {
    pub level: String,
}

/// Holds the config for the meshtastic node.
#[derive(Debug, Clone, Deserialize)]
pub struct MeshConfig {
    pub connection_type: ConnectionType,
    pub serial_port: Option<String>,
    pub ip: String,
    pub port: u16
}

/// Enum to define the connection type, defaulting to TCP
#[derive(Default, Debug, Clone, Deserialize)]
pub enum ConnectionType {
    #[default]
    Tcp,
    Serial,
    Mqtt,
}


// Defaults

impl Default for ServerConfig {
    fn default() -> Self {
        Self {
            host: "localhost".to_string(),
            port: 8080,
        }
    }
}

impl Default for DatabaseConfig {
    fn default() -> Self {
        Self {
            db_name: "yamm.db".to_string(),
        }
    }
}

impl Default for LoggingConfig {
    fn default() -> Self {
        Self {
            level: "INFO".to_string(),
        }
    }
}

impl Default for MeshConfig {
    fn default() -> Self {
        Self {
            connection_type: ConnectionType::Tcp,
            serial_port: None,
            ip: "192.168.1.2".to_string(),
            port: 4403,
        }
    }
}