use egui_extras::{Column, TableBuilder};
use tracing::debug;

use crate::db::Database;

pub(crate) async fn run_gui() {
    debug!("Running in GUI Mode!");
    use eframe::{App, egui};

    struct MyApp;

    impl App for MyApp {
        fn update(&mut self, ctx: &egui::Context, _frame: &mut eframe::Frame) {
            // Table Panel, showing nodes in a list.
            egui::CentralPanel::default().show(ctx, |ui| {
                // Get nodes from DB
                let nodes = Database::read_node_list().expect("Failed to read nodes");
                TableBuilder::new(ui)
                    .column(Column::auto().resizable(true))
                    .column(Column::remainder())
                    .column(Column::remainder())
                    .column(Column::remainder())
                    .column(Column::remainder())
                    .column(Column::remainder())
                    .column(Column::remainder())
                    .header(20.0, |mut header| {
                        header.col(|ui| {
                            ui.heading("Node ID");
                        });
                        header.col(|ui| {
                            ui.heading("Name");
                        });
                        header.col(|ui| {
                            ui.heading("SNR");
                        });
                        header.col(|ui| {
                            ui.heading("Hops Away");
                        });
                        header.col(|ui| {
                            ui.heading("HW Model");
                        });
                        header.col(|ui| {
                            ui.heading("Role");
                        });
                        header.col(|ui| {
                            ui.heading("Battery Level");
                        });
                    })
                    .body(|mut body| {
                        for node in nodes {
                            let user = node.user.unwrap();
                            let device = node.device_metrics.unwrap();
                            body.row(10.0, |mut row| {
                                row.col(|ui| {
                                    ui.label(user.id);
                                });
                                row.col(|ui| {
                                    ui.label(format!("{} ({})", user.long_name, user.short_name));
                                });
                                row.col(|ui| {
                                    ui.label(node.snr.to_string());
                                });
                                row.col(|ui| {
                                    ui.label(node.hops_away.to_string());
                                });
                                row.col(|ui| {
                                    ui.label(user.hw_model.to_string());
                                });
                                row.col(|ui| {
                                    ui.label(user.role.to_string());
                                });
                                row.col(|ui| {
                                    ui.label(device.battery_level.to_string());
                                });
                            });
                        }
                    });
            });
        }
    }

    let native_options = eframe::NativeOptions::default();
    eframe::run_native(
        "Yet Another Meshtastic Server (GUI)",
        native_options,
        Box::new(|_cc| Ok(Box::new(MyApp))),
    ).unwrap();
}